import { useBottomTabBarHeight } from "@react-navigation/bottom-tabs";
import React, { useEffect, useRef } from "react";
import { View, TouchableOpacity, Text, LayoutAnimation } from "react-native";
import Ionicons from "react-native-vector-icons/Ionicons";

export const CarouselTypeChanger = ({
  locationStatus,
  position,
  mapsRef,
  stationListMode,
  setStationListMode,
  setSelectedCurrentStation,
  mapMode,
  setMapMode,
}) => {
  const tabBarHeight = useBottomTabBarHeight();
  const returnToDefaultMode = ()=>{
    LayoutAnimation.configureNext({
      duration: 300,
      create: {
        type: LayoutAnimation.Types.easeInEaseOut,
        property: LayoutAnimation.Properties.opacity,
      },
      update: {
        type: LayoutAnimation.Types.easeInEaseOut,
        property: LayoutAnimation.Properties.opacity,
      },
    });
    setMapMode(false);
  }
  return (
    <View style={{ width: "100%", height: 40, flexDirection: "row", position: mapMode ? "absolute" : "relative", bottom: mapMode ? 0 : undefined}} key={"carouselTypeChanger"} >
      <TouchableOpacity
        style={{
          flex: 1,
          backgroundColor: "#0099CC",
          padding: 5,
          alignItems: "center",
          flexDirection: "row",
          marginHorizontal: 5,
          borderTopLeftRadius: 20,
          borderTopRightRadius: 20,
          borderBottomLeftRadius: stationListMode == "position" ? 0 : 20,
          borderBottomRightRadius: stationListMode == "position" ? 0 : 20,
        }}
        disabled={!locationStatus}
        
        onPressIn={() =>{
          if (!position) return;
          returnToDefaultMode();
          setStationListMode("position");
        }}
        onPress={() => {
          if (!position) return;
          returnToDefaultMode();
          setStationListMode("position");
        }}
      >
        <Ionicons
          name="locate-outline"
          size={14}
          color="white"
          style={{ margin: 5 }}
        />
        <Text
          style={{
            color: "white",
            fontSize: 14,
            fontWeight: "bold",
            flex: 1,
            textAlign: "center",
          }}
        >
          現在地基準
        </Text>
      </TouchableOpacity>
      <TouchableOpacity
        style={{
          padding: 5,
          alignItems: "center",
          flexDirection: "row",
          marginHorizontal: 5,
          borderRadius: 50,
        }}
        onPressIn={() => returnToDefaultMode()}
      >
        <Ionicons
          name={!mapMode ? "menu" : "chevron-up-outline"}
          size={30}
          color="#0099CC"
          style={{ marginHorizontal: 5 }}
        />
      </TouchableOpacity>
      <TouchableOpacity
        style={{
          flex: 1,
          backgroundColor: "#0099CC",
          padding: 5,
          alignItems: "center",
          flexDirection: "row",
          marginHorizontal: 5,
          borderTopLeftRadius: 20,
          borderTopRightRadius: 20,
          borderBottomLeftRadius: stationListMode == "favorite" ? 0 : 20,
          borderBottomRightRadius: stationListMode == "favorite" ? 0 : 20,
        }}
        onPressIn={() => {
          returnToDefaultMode();
          // お気に入りリスト更新
          setStationListMode("favorite");
          setSelectedCurrentStation(0);
        }}
        onPress={() => {
          returnToDefaultMode();
          // お気に入りリスト更新
          setStationListMode("favorite");
          setSelectedCurrentStation(0);
        }}
      >
        <Ionicons name="star" size={14} color="white" style={{ margin: 5 }} />
        <Text
          style={{
            color: "white",
            fontSize: 14,
            fontWeight: "bold",
            flex: 1,
            textAlign: "center",
          }}
        >
          お気に入りリスト
        </Text>
      </TouchableOpacity>
    </View>
  );
};
